import { ApolloServer } from "apollo-server";
import { Resolvers } from "./generated/graphql";
import { pizzaSchema } from "./schema/pizza";
import { PizzaResolver } from "./resolver/pizza-resolver";
import { ToppingResolver } from "./resolver/topping-resolver";



const resolvers: Resolvers = {
    Topping: {
        id: (root, args, context) => {
            return root.id
        },
        name: (root, args, context) => {
            return root.name
        },
    },
    Pizza: {
        id: (root, args, context) => {
            return root.id
        },
        name: (root, args, context) => {
            return root.name
        },
        toppings: (root, args, context) => {
            return root.toppings
        },
    },
    Query: {
        getPizzaById: (root, args, context) => {
            return PizzaResolver.getById(args.pizzaId);
        },
        getPizzaByName: (root, args, context) => {
            return PizzaResolver.getByName(args.pizzaName);
        },
        listPizza: (root, args, context) => {
            return PizzaResolver.list();
        },
        getToppingById: (root, args, context) => {
            return ToppingResolver.getById(args.toppingId);
        },
        getToppingByName: (root, args, context) => {
            return ToppingResolver.getByName(args.toppingName);
        },
        listTopping: (root, args, context) => {
            return ToppingResolver.list();
        },
    }
}

const server = new ApolloServer({
    typeDefs: pizzaSchema,
    resolvers: resolvers as any
});

server.listen().then(({ url }) => {
    console.log(`🚀 Server ready at ${url}`)
});